import { TypedEmitter } from "tiny-typed-emitter";
import sql from "better-sqlite3";
import { DatabaseEvents, DatabaseOptions, DataResolvable, SqliteQueryOptions, TableResolvable } from "../typings";
import { Table } from "./Table";
/**
 * Database wrapper class for sqlite.
 */
declare class Database extends TypedEmitter<DatabaseEvents> {
    /**
     * The sqlite database.
     */
    db?: sql.Database;
    /**
     * The options for this database.
     */
    options: DatabaseOptions;
    /**
     * The assigned tables for this database.
     */
    tables: Map<string, Table>;
    /**
     *
     * @param options The options for this database.
     */
    constructor(options?: DatabaseOptions);
    /**
     * Validates given options.
     * @param options The options to check.
     * @returns The options with their default values.
     */
    private _validateOptions;
    /**
     * Resolves a string or object into a table.
     * @param name The name of the table.
     * @param suppress Whether to suppress the error if table does not exist.
     * @returns The table found.
     */
    resolveTable(name: TableResolvable, suppress?: boolean): Table;
    /**
     * Creates a table.
     * @param name The name for this table
     * @returns The table that was created.
     */
    createTable(name: string): Table;
    /**
     * Returns the corresponding array from given map.
     * @param map The map to convert.
     * @returns The array from the map.
     */
    arrayFromMap(map: any): Table[];
    /**
     * Gets data from a table.
     * @param name The name of the table to get its data.
     * @returns The table data if it exists.
     */
    getTableData(name: string): Record<string, any> | undefined;
    /**
     * Gets all available tables from this database.
     * @returns An array of table data.
     */
    get databaseTables(): Record<string, any>[] | undefined;
    /**
     * Gets multiple rows from this database.
     * @param table The table to get rows from.
     * @param options The options to apply for this query.
     * @returns The rows found from this query.
     */
    all(table: TableResolvable, options?: SqliteQueryOptions): DataResolvable[];
    /**
     * Sets a row into the database (or updates existing one).
     * @param table The table to perform this action on.
     * @param data The data to set.
     * @param options Options for this query.
     * @returns Sqlite result for this setter.
     */
    set(table: TableResolvable, data: DataResolvable, options?: SqliteQueryOptions): (import("../typings").SqliteChanges | undefined)[];
    /**
     * Deletes rows from a database, if no options are given it will clean the entire table.
     * @param table The table to delete rows from.
     * @param options Options for this query.
     * @returns Affected rows and changes from the sqlite table.
     */
    delete(table: TableResolvable, options?: SqliteQueryOptions): import("../typings").SqliteChanges;
    /**
     * Insert one or multiple rows into the database.
     * If a row with given primary key exists, this will throw an error.
     * @param table The table to set these rows to.
     * @param data The row or rows to insert.
     * @returns The sqlite changes happened in the table.
     */
    insert(table: TableResolvable, data: DataResolvable[] | DataResolvable): import("../typings").SqliteChanges;
    /**
     * Gets a single row from the database.
     * @param table The table to get a row from.
     * @param options The options for this query.
     * @returns A row from the database, if it exists.
     * @example
     * db.get("mytable", {
     *      where: {
     *          column: "id",
     *          equals: "12345"
     *      }
     * })
     */
    get(table: TableResolvable, options: SqliteQueryOptions): DataResolvable | undefined;
    /**
     * Connects the database to its corresponding file.
     * @returns The connected database.
     */
    connect(): Promise<this>;
}
export default Database;
//# sourceMappingURL=Database.d.ts.map