"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Table = void 0;
const Util_1 = require("../util/Util");
const Column_1 = require("./Column");
class Table {
    constructor(name, db, columns) {
        this._columns = new Array();
        this.columns = new Array();
        this.ready = false;
        this.db = db;
        this.name = name;
        if (columns) {
            this._columns = columns;
        }
    }
    setDatabase(db) {
        this.db = db;
        return this;
    }
    setName(str) {
        this.name = str;
        return this;
    }
    transformColumns() {
        return this._columns.map(c => Column_1.Column.transform(c, this.db));
    }
    getTypeColumns(type) {
        return this.columns.filter(c => c.type === type);
    }
    getColumns() {
        return this.columns;
    }
    getColumn(index) {
        return this.columns[index];
    }
    isReady() {
        return this.ready;
    }
    get columnCreator() {
        return this.columns.map(c => c.creator).join(", ");
    }
    getTableColumns() {
        var _a;
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.pragma(`table_info(${this.name})`);
    }
    create() {
        var _a;
        const got = this.db.getTableData(this.name);
        if (got) {
            const columns = this.getTableColumns();
            this.transformColumns().map(c => this.columns.push(c));
            for (const column of this.columns) {
                const clm = columns.find(c => c.name === column.name);
                if (!clm) {
                    column.create(this);
                }
            }
            return false;
        }
        this.transformColumns().map(c => this.columns.push(c));
        const e = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`CREATE TABLE ${this.name}(${this.columnCreator})`).run();
        return true;
    }
    addColumns(columns, force = false) {
        columns.map(c => this.addColumn(c, force));
        return this;
    }
    addColumn(column, force = false) {
        var _a;
        this._columns.push(column);
        if (force) {
            (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`ALTER TABLE ${this.name} ADD COLUMN ${column.name}`).run();
            this.columns.push(Column_1.Column.transform(column, this.db));
        }
        return this;
    }
    get(options) {
        var _a, _b;
        const opts = Util_1.Util.makeWhereOptions(options);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT ${(_b = options.columns) !== null && _b !== void 0 ? _b : "*"} FROM ${this.name} ${opts}`).get();
        if (this.db.options.sanitize) {
            for (const column of this.getTypeColumns("JSON")) {
                if (data[column.name] !== null) {
                    data[column.name] = JSON.parse(data[column.name]);
                }
            }
            for (const column of this.getTypeColumns("BOOLEAN")) {
                if (data[column.name] !== null) {
                    data[column.name] = data[column.name] === 1;
                }
            }
        }
        return data;
    }
    all(options) {
        var _a, _b;
        const opts = Util_1.Util.makeWhereOptions(options);
        const data = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`SELECT ${(_b = options === null || options === void 0 ? void 0 : options.columns) !== null && _b !== void 0 ? _b : "*"} FROM ${this.name} ${opts}`).all();
        if (this.db.options.sanitize && this.db.options.length !== undefined) {
            if (this.db.options.length > 0 && data.length <= this.db.options.length) {
                for (let i = 0; i < data.length; i++) {
                    const d = data[i];
                    for (const column of this.getTypeColumns("JSON")) {
                        if (d[column.name] !== null) {
                            d[column.name] = JSON.parse(d[column.name]);
                        }
                    }
                    for (const column of this.getTypeColumns("BOOLEAN")) {
                        if (d[column.name] !== null) {
                            d[column.name] = d[column.name] === 1;
                        }
                    }
                }
            }
        }
        return data;
    }
    insert(data) {
        var _a;
        const datas = Array.isArray(data) ? data : [data];
        const columns = Util_1.Util.makeColumnsFrom(datas[0]);
        const values = datas.map(d => `(${Util_1.Util.makeInsertValues(d, this.columns)})`);
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`INSERT INTO ${this.name}(${columns}) VALUES ${values.join(", \n")}`).run();
    }
    delete(options) {
        var _a;
        const opts = Util_1.Util.makeWhereOptions(options);
        return (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`DELETE FROM ${this.name} ${opts}`).run();
    }
    set(data, options) {
        var _a, _b;
        const opts = Util_1.Util.makeWhereOptions(options);
        const values = Util_1.Util.makeSetValues(data, this.columns);
        const insert = Util_1.Util.makeInsertValues(data, this.columns);
        const columns = Util_1.Util.makeColumnsFrom(data);
        const ch1 = (_a = this.db.db) === null || _a === void 0 ? void 0 : _a.prepare(`INSERT OR IGNORE INTO ${this.name} (${columns}) VALUES (${insert})`).run();
        const ch2 = (_b = this.db.db) === null || _b === void 0 ? void 0 : _b.prepare(`UPDATE ${this.name} SET ${values} ${opts}`).run();
        return [
            ch1,
            ch2
        ];
    }
}
exports.Table = Table;
//# sourceMappingURL=Table.js.map